/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.reuseware.coconut.reuseextension.resource.rex;

/**
 * An element that is expected at a given position in a resource stream.
 */
public interface IRexExpectedElement {
	
	/**
	 * Returns the names of all tokens that are expected at the given position.
	 */
	public java.util.Set<String> getTokenNames();
	
	/**
	 * Returns the metaclass of the rule that contains the expected element.
	 */
	public org.eclipse.emf.ecore.EClass getRuleMetaclass();
	
	/**
	 * Returns the syntax element that is expected.
	 */
	public org.reuseware.coconut.reuseextension.resource.rex.grammar.RexSyntaxElement getSymtaxElement();
	
	/**
	 * Adds an element that is a valid follower for this element.
	 */
	public void addFollower(org.reuseware.coconut.reuseextension.resource.rex.IRexExpectedElement follower, org.reuseware.coconut.reuseextension.resource.rex.mopp.RexContainedFeature[] path);
	
	/**
	 * Returns all valid followers for this element. Each follower is represented by a
	 * pair of an expected elements and the containment trace that leads from the
	 * current element to the follower.
	 */
	public java.util.Collection<org.reuseware.coconut.reuseextension.resource.rex.util.RexPair<org.reuseware.coconut.reuseextension.resource.rex.IRexExpectedElement, org.reuseware.coconut.reuseextension.resource.rex.mopp.RexContainedFeature[]>> getFollowers();
	
}
