/**
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 *  
 */
package org.reuseware.coconut.reuseextension.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import org.eclipse.emf.ecore.util.Switch;

import org.reuseware.coconut.reuseextension.*;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see org.reuseware.coconut.reuseextension.ReuseextensionPackage
 * @generated
 */
public class ReuseextensionSwitch<T> extends Switch<T>
{
  /**
   * The cached model package
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected static ReuseextensionPackage modelPackage;

  /**
   * Creates an instance of the switch.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  public ReuseextensionSwitch()
  {
    if (modelPackage == null)
    {
      modelPackage = ReuseextensionPackage.eINSTANCE;
    }
  }

  /**
   * Checks whether this is a switch for the given package.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @parameter ePackage the package in question.
   * @return whether this is a switch for the given package.
   * @generated
   */
  @Override
  protected boolean isSwitchFor(EPackage ePackage)
  {
    return ePackage == modelPackage;
  }

  /**
   * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @return the first non-null result returned by a <code>caseXXX</code> call.
   * @generated
   */
  @Override
  protected T doSwitch(int classifierID, EObject theEObject)
  {
    switch (classifierID)
    {
      case ReuseextensionPackage.ADDRESSABLE_POINT_DERIVATION_RULE:
      {
        AddressablePointDerivationRule addressablePointDerivationRule = (AddressablePointDerivationRule)theEObject;
        T result = caseAddressablePointDerivationRule(addressablePointDerivationRule);
        if (result == null) result = caseRuleContext(addressablePointDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.ANCHOR_DERIVATION_RULE:
      {
        AnchorDerivationRule anchorDerivationRule = (AnchorDerivationRule)theEObject;
        T result = caseAnchorDerivationRule(anchorDerivationRule);
        if (result == null) result = caseReferencePointDerivationRule(anchorDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(anchorDerivationRule);
        if (result == null) result = caseRuleContext(anchorDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION:
      {
        ComponentModelSpecification componentModelSpecification = (ComponentModelSpecification)theEObject;
        T result = caseComponentModelSpecification(componentModelSpecification);
        if (result == null) result = caseReuseExtension(componentModelSpecification);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING:
      {
        CompositionAssociation2CompositionLinkBinding compositionAssociation2CompositionLinkBinding = (CompositionAssociation2CompositionLinkBinding)theEObject;
        T result = caseCompositionAssociation2CompositionLinkBinding(compositionAssociation2CompositionLinkBinding);
        if (result == null) result = caseCompositionAssociationBinding(compositionAssociation2CompositionLinkBinding);
        if (result == null) result = caseRuleContext(compositionAssociation2CompositionLinkBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.COMPOSITION_ASSOCIATION_BINDING:
      {
        CompositionAssociationBinding compositionAssociationBinding = (CompositionAssociationBinding)theEObject;
        T result = caseCompositionAssociationBinding(compositionAssociationBinding);
        if (result == null) result = caseRuleContext(compositionAssociationBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION:
      {
        CompositionLanguageSpecification compositionLanguageSpecification = (CompositionLanguageSpecification)theEObject;
        T result = caseCompositionLanguageSpecification(compositionLanguageSpecification);
        if (result == null) result = caseReuseExtension(compositionLanguageSpecification);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION:
      {
        CompositionLanguageSyntaxSpecification compositionLanguageSyntaxSpecification = (CompositionLanguageSyntaxSpecification)theEObject;
        T result = caseCompositionLanguageSyntaxSpecification(compositionLanguageSyntaxSpecification);
        if (result == null) result = caseReuseExtension(compositionLanguageSyntaxSpecification);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING:
      {
        FragmentRole2FragmentBinding fragmentRole2FragmentBinding = (FragmentRole2FragmentBinding)theEObject;
        T result = caseFragmentRole2FragmentBinding(fragmentRole2FragmentBinding);
        if (result == null) result = caseFragmentRoleBinding(fragmentRole2FragmentBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING:
      {
        FragmentRole2FragmentInstanceBinding fragmentRole2FragmentInstanceBinding = (FragmentRole2FragmentInstanceBinding)theEObject;
        T result = caseFragmentRole2FragmentInstanceBinding(fragmentRole2FragmentInstanceBinding);
        if (result == null) result = caseFragmentRoleBinding(fragmentRole2FragmentInstanceBinding);
        if (result == null) result = caseRuleContext(fragmentRole2FragmentInstanceBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING:
      {
        FragmentRole2FragmentReferenceBinding fragmentRole2FragmentReferenceBinding = (FragmentRole2FragmentReferenceBinding)theEObject;
        T result = caseFragmentRole2FragmentReferenceBinding(fragmentRole2FragmentReferenceBinding);
        if (result == null) result = caseFragmentRole2FragmentInstanceBinding(fragmentRole2FragmentReferenceBinding);
        if (result == null) result = caseFragmentRoleBinding(fragmentRole2FragmentReferenceBinding);
        if (result == null) result = caseRuleContext(fragmentRole2FragmentReferenceBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING:
      {
        FragmentRole2SyntaxBinding fragmentRole2SyntaxBinding = (FragmentRole2SyntaxBinding)theEObject;
        T result = caseFragmentRole2SyntaxBinding(fragmentRole2SyntaxBinding);
        if (result == null) result = caseFragmentRoleBinding(fragmentRole2SyntaxBinding);
        if (result == null) result = caseSyntaxDerivationRule(fragmentRole2SyntaxBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.FRAGMENT_ROLE_BINDING:
      {
        FragmentRoleBinding fragmentRoleBinding = (FragmentRoleBinding)theEObject;
        T result = caseFragmentRoleBinding(fragmentRoleBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.HOOK_DERIVATION_RULE:
      {
        HookDerivationRule hookDerivationRule = (HookDerivationRule)theEObject;
        T result = caseHookDerivationRule(hookDerivationRule);
        if (result == null) result = caseVariationPointDerivationRule(hookDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(hookDerivationRule);
        if (result == null) result = caseRuleContext(hookDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING:
      {
        PortType2HeterogeneousPortBinding portType2HeterogeneousPortBinding = (PortType2HeterogeneousPortBinding)theEObject;
        T result = casePortType2HeterogeneousPortBinding(portType2HeterogeneousPortBinding);
        if (result == null) result = casePortType2PortBinding(portType2HeterogeneousPortBinding);
        if (result == null) result = casePortTypeBinding(portType2HeterogeneousPortBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING:
      {
        PortType2HomogenousPortBinding portType2HomogenousPortBinding = (PortType2HomogenousPortBinding)theEObject;
        T result = casePortType2HomogenousPortBinding(portType2HomogenousPortBinding);
        if (result == null) result = casePortType2PortBinding(portType2HomogenousPortBinding);
        if (result == null) result = casePortTypeBinding(portType2HomogenousPortBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.PORT_TYPE2_PORT_BINDING:
      {
        PortType2PortBinding portType2PortBinding = (PortType2PortBinding)theEObject;
        T result = casePortType2PortBinding(portType2PortBinding);
        if (result == null) result = casePortTypeBinding(portType2PortBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING:
      {
        PortType2SettingBinding portType2SettingBinding = (PortType2SettingBinding)theEObject;
        T result = casePortType2SettingBinding(portType2SettingBinding);
        if (result == null) result = casePortTypeBinding(portType2SettingBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING:
      {
        PortType2SyntaxBinding portType2SyntaxBinding = (PortType2SyntaxBinding)theEObject;
        T result = casePortType2SyntaxBinding(portType2SyntaxBinding);
        if (result == null) result = caseSyntaxDerivationRule(portType2SyntaxBinding);
        if (result == null) result = casePortTypeBinding(portType2SyntaxBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.PORT_TYPE_BINDING:
      {
        PortTypeBinding portTypeBinding = (PortTypeBinding)theEObject;
        T result = casePortTypeBinding(portTypeBinding);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE:
      {
        PrototypeDerivationRule prototypeDerivationRule = (PrototypeDerivationRule)theEObject;
        T result = casePrototypeDerivationRule(prototypeDerivationRule);
        if (result == null) result = caseReferencePointDerivationRule(prototypeDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(prototypeDerivationRule);
        if (result == null) result = caseRuleContext(prototypeDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.REFERENCE_POINT_DERIVATION_RULE:
      {
        ReferencePointDerivationRule referencePointDerivationRule = (ReferencePointDerivationRule)theEObject;
        T result = caseReferencePointDerivationRule(referencePointDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(referencePointDerivationRule);
        if (result == null) result = caseRuleContext(referencePointDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.REUSE_EXTENSION:
      {
        ReuseExtension reuseExtension = (ReuseExtension)theEObject;
        T result = caseReuseExtension(reuseExtension);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.ROOT_ELEMENT_CONTEXT:
      {
        RootElementContext rootElementContext = (RootElementContext)theEObject;
        T result = caseRootElementContext(rootElementContext);
        if (result == null) result = caseRuleContext(rootElementContext);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.RULE_CONTEXT:
      {
        RuleContext ruleContext = (RuleContext)theEObject;
        T result = caseRuleContext(ruleContext);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.SETTING_DERIVATION_RULE:
      {
        SettingDerivationRule settingDerivationRule = (SettingDerivationRule)theEObject;
        T result = caseSettingDerivationRule(settingDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.SLOT_DERIVATION_RULE:
      {
        SlotDerivationRule slotDerivationRule = (SlotDerivationRule)theEObject;
        T result = caseSlotDerivationRule(slotDerivationRule);
        if (result == null) result = caseVariationPointDerivationRule(slotDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(slotDerivationRule);
        if (result == null) result = caseRuleContext(slotDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.SYNTAX_DERIVATION_RULE:
      {
        SyntaxDerivationRule syntaxDerivationRule = (SyntaxDerivationRule)theEObject;
        T result = caseSyntaxDerivationRule(syntaxDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE:
      {
        ValueHookDerivationRule valueHookDerivationRule = (ValueHookDerivationRule)theEObject;
        T result = caseValueHookDerivationRule(valueHookDerivationRule);
        if (result == null) result = caseVariationPointDerivationRule(valueHookDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(valueHookDerivationRule);
        if (result == null) result = caseRuleContext(valueHookDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE:
      {
        ValuePrototypeDerivationRule valuePrototypeDerivationRule = (ValuePrototypeDerivationRule)theEObject;
        T result = caseValuePrototypeDerivationRule(valuePrototypeDerivationRule);
        if (result == null) result = caseReferencePointDerivationRule(valuePrototypeDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(valuePrototypeDerivationRule);
        if (result == null) result = caseRuleContext(valuePrototypeDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case ReuseextensionPackage.VARIATION_POINT_DERIVATION_RULE:
      {
        VariationPointDerivationRule variationPointDerivationRule = (VariationPointDerivationRule)theEObject;
        T result = caseVariationPointDerivationRule(variationPointDerivationRule);
        if (result == null) result = caseAddressablePointDerivationRule(variationPointDerivationRule);
        if (result == null) result = caseRuleContext(variationPointDerivationRule);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      default: return defaultCase(theEObject);
    }
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Addressable Point Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Addressable Point Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseAddressablePointDerivationRule(AddressablePointDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Anchor Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Anchor Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseAnchorDerivationRule(AnchorDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Component Model Specification</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Component Model Specification</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseComponentModelSpecification(ComponentModelSpecification object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Composition Association2 Composition Link Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Composition Association2 Composition Link Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseCompositionAssociation2CompositionLinkBinding(CompositionAssociation2CompositionLinkBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Composition Association Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Composition Association Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseCompositionAssociationBinding(CompositionAssociationBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Composition Language Specification</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Composition Language Specification</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseCompositionLanguageSpecification(CompositionLanguageSpecification object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Composition Language Syntax Specification</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Composition Language Syntax Specification</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseCompositionLanguageSyntaxSpecification(CompositionLanguageSyntaxSpecification object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Fragment Role2 Fragment Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Fragment Role2 Fragment Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseFragmentRole2FragmentBinding(FragmentRole2FragmentBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Fragment Role2 Fragment Instance Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Fragment Role2 Fragment Instance Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseFragmentRole2FragmentInstanceBinding(FragmentRole2FragmentInstanceBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Fragment Role2 Fragment Reference Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Fragment Role2 Fragment Reference Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseFragmentRole2FragmentReferenceBinding(FragmentRole2FragmentReferenceBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Fragment Role2 Syntax Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Fragment Role2 Syntax Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseFragmentRole2SyntaxBinding(FragmentRole2SyntaxBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Fragment Role Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Fragment Role Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseFragmentRoleBinding(FragmentRoleBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Hook Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Hook Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseHookDerivationRule(HookDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Port Type2 Heterogeneous Port Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Port Type2 Heterogeneous Port Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePortType2HeterogeneousPortBinding(PortType2HeterogeneousPortBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Port Type2 Homogenous Port Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Port Type2 Homogenous Port Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePortType2HomogenousPortBinding(PortType2HomogenousPortBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Port Type2 Port Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Port Type2 Port Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePortType2PortBinding(PortType2PortBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Port Type2 Setting Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Port Type2 Setting Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePortType2SettingBinding(PortType2SettingBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Port Type2 Syntax Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Port Type2 Syntax Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePortType2SyntaxBinding(PortType2SyntaxBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Port Type Binding</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Port Type Binding</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePortTypeBinding(PortTypeBinding object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Prototype Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Prototype Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T casePrototypeDerivationRule(PrototypeDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Reference Point Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Reference Point Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseReferencePointDerivationRule(ReferencePointDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Reuse Extension</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Reuse Extension</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseReuseExtension(ReuseExtension object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Root Element Context</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Root Element Context</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRootElementContext(RootElementContext object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Rule Context</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Rule Context</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRuleContext(RuleContext object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Setting Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Setting Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseSettingDerivationRule(SettingDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Slot Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Slot Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseSlotDerivationRule(SlotDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Syntax Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Syntax Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseSyntaxDerivationRule(SyntaxDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Value Hook Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Value Hook Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseValueHookDerivationRule(ValueHookDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Value Prototype Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Value Prototype Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseValuePrototypeDerivationRule(ValuePrototypeDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Variation Point Derivation Rule</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Variation Point Derivation Rule</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseVariationPointDerivationRule(VariationPointDerivationRule object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch, but this is the last case anyway.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject)
   * @generated
   */
  @Override
  public T defaultCase(EObject object)
  {
    return null;
  }

} //ReuseextensionSwitch
