/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.mopp;

/**
 * The Rex_activatorTokenResolverFactory class provides access to all generated
 * token resolvers. By giving the name of a defined token, the corresponding
 * resolve can be obtained. Despite the fact that this class is called
 * TokenResolverFactory is does NOT create new token resolvers whenever a client
 * calls methods to obtain a resolver. Rather, this class maintains a map of all
 * resolvers and creates each resolver at most once.
 */
public class Rex_activatorTokenResolverFactory implements org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolverFactory {
	
	private java.util.Map<String, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver> tokenName2TokenResolver;
	private java.util.Map<String, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver> featureName2CollectInTokenResolver;
	private static org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver defaultResolver = new org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.analysis.Rex_activatorDefaultTokenResolver();
	
	public Rex_activatorTokenResolverFactory() {
		tokenName2TokenResolver = new java.util.LinkedHashMap<String, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver>();
		featureName2CollectInTokenResolver = new java.util.LinkedHashMap<String, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver>();
		registerTokenResolver("TEXT", new org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.analysis.Rex_activatorTEXTTokenResolver());
		registerTokenResolver("QUOTED_39_39", new org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.analysis.Rex_activatorQUOTED_39_39TokenResolver());
	}
	
	public org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver createTokenResolver(String tokenName) {
		return internalCreateResolver(tokenName2TokenResolver, tokenName);
	}
	
	public org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver createCollectInTokenResolver(String featureName) {
		return internalCreateResolver(featureName2CollectInTokenResolver, featureName);
	}
	
	protected boolean registerTokenResolver(String tokenName, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver resolver){
		return internalRegisterTokenResolver(tokenName2TokenResolver, tokenName, resolver);
	}
	
	protected boolean registerCollectInTokenResolver(String featureName, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver resolver){
		return internalRegisterTokenResolver(featureName2CollectInTokenResolver, featureName, resolver);
	}
	
	protected org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver deRegisterTokenResolver(String tokenName){
		return tokenName2TokenResolver.remove(tokenName);
	}
	
	private org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver internalCreateResolver(java.util.Map<String, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver> resolverMap, String key) {
		if (resolverMap.containsKey(key)){
			return resolverMap.get(key);
		} else {
			return defaultResolver;
		}
	}
	
	private boolean internalRegisterTokenResolver(java.util.Map<String, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver> resolverMap, String key, org.reuseware.coconut.reuseextensionactivator.resource.rexactivator.IRex_activatorTokenResolver resolver) {
		if (!resolverMap.containsKey(key)) {
			resolverMap.put(key,resolver);
			return true;
		}
		return false;
	}
	
}
