/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.reuseware.coconut.reuseextension.resource.rex.mopp;

public class RexPrinter implements org.reuseware.coconut.reuseextension.resource.rex.IRexTextPrinter {
	
	protected org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolverFactory tokenResolverFactory = new org.reuseware.coconut.reuseextension.resource.rex.mopp.RexTokenResolverFactory();
	
	protected java.io.OutputStream outputStream;
	
	/**
	 * Holds the resource that is associated with this printer. This may be null if
	 * the printer is used stand alone.
	 */
	private org.reuseware.coconut.reuseextension.resource.rex.IRexTextResource resource;
	
	private java.util.Map<?, ?> options;
	private String encoding = System.getProperty("file.encoding");
	
	public RexPrinter(java.io.OutputStream outputStream, org.reuseware.coconut.reuseextension.resource.rex.IRexTextResource resource) {
		super();
		this.outputStream = outputStream;
		this.resource = resource;
	}
	
	protected int matchCount(java.util.Map<String, Integer> featureCounter, java.util.Collection<String> needed) {
		int pos = 0;
		int neg = 0;
		
		for (String featureName : featureCounter.keySet()) {
			if (needed.contains(featureName)) {
				int value = featureCounter.get(featureName);
				if (value == 0) {
					neg += 1;
				} else {
					pos += 1;
				}
			}
		}
		return neg > 0 ? -neg : pos;
	}
	
	protected void doPrint(org.eclipse.emf.ecore.EObject element, java.io.PrintWriter out, String globaltab) {
		if (element == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write.");
		}
		if (out == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write on.");
		}
		
		if (element instanceof org.reuseware.coconut.reuseextension.ComponentModelSpecification) {
			print_org_reuseware_coconut_reuseextension_ComponentModelSpecification((org.reuseware.coconut.reuseextension.ComponentModelSpecification) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification) {
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification((org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.CompositionLanguageSpecification) {
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification((org.reuseware.coconut.reuseextension.CompositionLanguageSpecification) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.RootElementContext) {
			print_org_reuseware_coconut_reuseextension_RootElementContext((org.reuseware.coconut.reuseextension.RootElementContext) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.FragmentRole2FragmentBinding) {
			print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentBinding((org.reuseware.coconut.reuseextension.FragmentRole2FragmentBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.PortType2HeterogeneousPortBinding) {
			print_org_reuseware_coconut_reuseextension_PortType2HeterogeneousPortBinding((org.reuseware.coconut.reuseextension.PortType2HeterogeneousPortBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.PortType2HomogenousPortBinding) {
			print_org_reuseware_coconut_reuseextension_PortType2HomogenousPortBinding((org.reuseware.coconut.reuseextension.PortType2HomogenousPortBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding) {
			print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding((org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding) {
			print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding((org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.PortType2SettingBinding) {
			print_org_reuseware_coconut_reuseextension_PortType2SettingBinding((org.reuseware.coconut.reuseextension.PortType2SettingBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.SettingDerivationRule) {
			print_org_reuseware_coconut_reuseextension_SettingDerivationRule((org.reuseware.coconut.reuseextension.SettingDerivationRule) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding) {
			print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding((org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.PortType2SyntaxBinding) {
			print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding((org.reuseware.coconut.reuseextension.PortType2SyntaxBinding) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.SlotDerivationRule) {
			print_org_reuseware_coconut_reuseextension_SlotDerivationRule((org.reuseware.coconut.reuseextension.SlotDerivationRule) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.HookDerivationRule) {
			print_org_reuseware_coconut_reuseextension_HookDerivationRule((org.reuseware.coconut.reuseextension.HookDerivationRule) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.AnchorDerivationRule) {
			print_org_reuseware_coconut_reuseextension_AnchorDerivationRule((org.reuseware.coconut.reuseextension.AnchorDerivationRule) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.PrototypeDerivationRule) {
			print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule((org.reuseware.coconut.reuseextension.PrototypeDerivationRule) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.ValueHookDerivationRule) {
			print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule((org.reuseware.coconut.reuseextension.ValueHookDerivationRule) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule) {
			print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule((org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule) element, globaltab, out);
			return;
		}
		if (element instanceof org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding) {
			print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding((org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding) element, globaltab, out);
			return;
		}
		
		addWarningToResource("The printer can not handle " + element.eClass().getName() + " elements", element);
	}
	
	protected org.reuseware.coconut.reuseextension.resource.rex.mopp.RexReferenceResolverSwitch getReferenceResolverSwitch() {
		return (org.reuseware.coconut.reuseextension.resource.rex.mopp.RexReferenceResolverSwitch) new org.reuseware.coconut.reuseextension.resource.rex.mopp.RexMetaInformation().getReferenceResolverSwitch();
	}
	
	protected void addWarningToResource(final String errorMessage, org.eclipse.emf.ecore.EObject cause) {
		org.reuseware.coconut.reuseextension.resource.rex.IRexTextResource resource = getResource();
		if (resource == null) {
			// the resource can be null if the printer is used stand alone
			return;
		}
		resource.addProblem(new org.reuseware.coconut.reuseextension.resource.rex.mopp.RexProblem(errorMessage, org.reuseware.coconut.reuseextension.resource.rex.RexEProblemType.PRINT_PROBLEM, org.reuseware.coconut.reuseextension.resource.rex.RexEProblemSeverity.WARNING), cause);
	}
	
	public void setOptions(java.util.Map<?,?> options) {
		this.options = options;
	}
	
	public java.util.Map<?,?> getOptions() {
		return options;
	}
	
	public void setEncoding(String encoding) {
		if (encoding != null) {
			this.encoding = encoding;
		}
	}
	
	public String getEncoding() {
		return encoding;
	}
	
	public org.reuseware.coconut.reuseextension.resource.rex.IRexTextResource getResource() {
		return resource;
	}
	
	/**
	 * Calls {@link #doPrint(EObject, PrintWriter, String)} and writes the result to
	 * the underlying output stream.
	 */
	public void print(org.eclipse.emf.ecore.EObject element) throws java.io.IOException {
		java.io.PrintWriter out = new java.io.PrintWriter(new java.io.OutputStreamWriter(new java.io.BufferedOutputStream(outputStream), encoding));
		doPrint(element, out, "");
		out.flush();
		out.close();
	}
	
	public void print_org_reuseware_coconut_reuseextension_ComponentModelSpecification(org.reuseware.coconut.reuseextension.ComponentModelSpecification element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(8);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__EPACKAGES));
		printCountingMap.put("ePackages", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRACOL_NAMESPACE));
		printCountingMap.put("fracolNamespace", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRACOL));
		printCountingMap.put("fracol", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__PARAMETERS));
		printCountingMap.put("parameters", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__REX_NAME));
		printCountingMap.put("rexName", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__REX_NAMESPACE));
		printCountingMap.put("rexNamespace", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__ROOT_ELEMENT_CONTEXTS));
		printCountingMap.put("rootElementContexts", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRAGMENT_ROLE_BINDINGS));
		printCountingMap.put("fragmentRoleBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("componentmodel");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("rexName");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__REX_NAME));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__REX_NAME), element));
				out.print(" ");
			}
			printCountingMap.put("rexName", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("implements");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_2(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracol");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRACOL));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getReuseExtensionFracolReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentCollaboration) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRACOL)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRACOL), element));
				out.print(" ");
			}
			printCountingMap.put("fracol", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("epackages");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("ePackages");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__EPACKAGES));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_60_62");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getReuseExtensionEPackagesReferenceResolver().deResolve((org.eclipse.emf.ecore.EPackage) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__EPACKAGES)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__EPACKAGES), element));
				out.print(" ");
			}
			printCountingMap.put("ePackages", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("rootclass");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("rootElementContexts");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__ROOT_ELEMENT_CONTEXTS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("rootElementContexts", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_3(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("fragmentRoleBindings");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRAGMENT_ROLE_BINDINGS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("fragmentRoleBindings", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_0(org.reuseware.coconut.reuseextension.ComponentModelSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("rexNamespace");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__REX_NAMESPACE));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__REX_NAMESPACE), element));
				out.print(" ");
			}
			printCountingMap.put("rexNamespace", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_1(org.reuseware.coconut.reuseextension.ComponentModelSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("(");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("parameters");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__PARAMETERS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__PARAMETERS), element));
				out.print(" ");
			}
			printCountingMap.put("parameters", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_1_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(")");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_1_0(org.reuseware.coconut.reuseextension.ComponentModelSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(",");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("parameters");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__PARAMETERS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__PARAMETERS), element));
				out.print(" ");
			}
			printCountingMap.put("parameters", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_2(org.reuseware.coconut.reuseextension.ComponentModelSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracolNamespace");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRACOL_NAMESPACE));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__FRACOL_NAMESPACE), element));
				out.print(" ");
			}
			printCountingMap.put("fracolNamespace", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_ComponentModelSpecification_3(org.reuseware.coconut.reuseextension.ComponentModelSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(",");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("rootElementContexts");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPONENT_MODEL_SPECIFICATION__ROOT_ELEMENT_CONTEXTS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("rootElementContexts", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification(org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(8);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__EPACKAGES));
		printCountingMap.put("ePackages", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRACOL_NAMESPACE));
		printCountingMap.put("fracolNamespace", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRACOL));
		printCountingMap.put("fracol", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__PARAMETERS));
		printCountingMap.put("parameters", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__REX_NAME));
		printCountingMap.put("rexName", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__REX_NAMESPACE));
		printCountingMap.put("rexNamespace", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__ROOT_ELEMENT_CONTEXTS));
		printCountingMap.put("rootElementContexts", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRAGMENT_ROLE_BINDINGS));
		printCountingMap.put("fragmentRoleBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("compositionlanguagesyntax");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("rexName");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__REX_NAME));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__REX_NAME), element));
				out.print(" ");
			}
			printCountingMap.put("rexName", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("implements");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_2(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracol");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRACOL));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getReuseExtensionFracolReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentCollaboration) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRACOL)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRACOL), element));
				out.print(" ");
			}
			printCountingMap.put("fracol", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("fragmentRoleBindings");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRAGMENT_ROLE_BINDINGS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("fragmentRoleBindings", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_0(org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("rexNamespace");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__REX_NAMESPACE));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__REX_NAMESPACE), element));
				out.print(" ");
			}
			printCountingMap.put("rexNamespace", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_1(org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("(");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("parameters");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__PARAMETERS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__PARAMETERS), element));
				out.print(" ");
			}
			printCountingMap.put("parameters", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_1_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(")");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_1_0(org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(",");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("parameters");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__PARAMETERS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__PARAMETERS), element));
				out.print(" ");
			}
			printCountingMap.put("parameters", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSyntaxSpecification_2(org.reuseware.coconut.reuseextension.CompositionLanguageSyntaxSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracolNamespace");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRACOL_NAMESPACE));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SYNTAX_SPECIFICATION__FRACOL_NAMESPACE), element));
				out.print(" ");
			}
			printCountingMap.put("fracolNamespace", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(11);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__EPACKAGES));
		printCountingMap.put("ePackages", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRACOL_NAMESPACE));
		printCountingMap.put("fracolNamespace", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRACOL));
		printCountingMap.put("fracol", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__PARAMETERS));
		printCountingMap.put("parameters", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__REX_NAME));
		printCountingMap.put("rexName", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__REX_NAMESPACE));
		printCountingMap.put("rexNamespace", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__ROOT_ELEMENT_CONTEXTS));
		printCountingMap.put("rootElementContexts", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__COMPOSITION_ASSOCIATION_BINDINGS));
		printCountingMap.put("compositionAssociationBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__COMPOSITION_PROGRAM_PHYSICAL_LOCATION_EXPRESSION));
		printCountingMap.put("compositionProgramPhysicalLocationExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRAGMENT_ROLE_BINDINGS));
		printCountingMap.put("fragmentRoleBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__UCPI_EXPRESSION));
		printCountingMap.put("ucpiExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("compositionlanguage");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("rexName");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__REX_NAME));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__REX_NAME), element));
				out.print(" ");
			}
			printCountingMap.put("rexName", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("implements");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_2(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracol");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRACOL));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getReuseExtensionFracolReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentCollaboration) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRACOL)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRACOL), element));
				out.print(" ");
			}
			printCountingMap.put("fracol", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("epackages");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("ePackages");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__EPACKAGES));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_60_62");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getReuseExtensionEPackagesReferenceResolver().deResolve((org.eclipse.emf.ecore.EPackage) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__EPACKAGES)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__EPACKAGES), element));
				out.print(" ");
			}
			printCountingMap.put("ePackages", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("rootclass");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("rootElementContexts");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__ROOT_ELEMENT_CONTEXTS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("rootElementContexts", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_3(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("ucpi");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("ucpiExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__UCPI_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__UCPI_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("ucpiExpression", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_5(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_0(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("rexNamespace");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__REX_NAMESPACE));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__REX_NAMESPACE), element));
				out.print(" ");
			}
			printCountingMap.put("rexNamespace", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_1(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("(");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("parameters");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__PARAMETERS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__PARAMETERS), element));
				out.print(" ");
			}
			printCountingMap.put("parameters", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_1_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(")");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_1_0(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(",");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("parameters");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__PARAMETERS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__PARAMETERS), element));
				out.print(" ");
			}
			printCountingMap.put("parameters", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_2(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fracolNamespace");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRACOL_NAMESPACE));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRACOL_NAMESPACE), element));
				out.print(" ");
			}
			printCountingMap.put("fracolNamespace", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_3(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(",");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("rootElementContexts");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__ROOT_ELEMENT_CONTEXTS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("rootElementContexts", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_4(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("location");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("compositionProgramPhysicalLocationExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__COMPOSITION_PROGRAM_PHYSICAL_LOCATION_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__COMPOSITION_PROGRAM_PHYSICAL_LOCATION_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("compositionProgramPhysicalLocationExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionLanguageSpecification_5(org.reuseware.coconut.reuseextension.CompositionLanguageSpecification element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		int alt = -1;
		alt = 0;
		int matches = 		matchCount(printCountingMap, java.util.Arrays.asList(		"fragmentRoleBindings"		));
		int tempMatchCount;
		tempMatchCount = 		matchCount(printCountingMap, java.util.Arrays.asList(		"compositionAssociationBindings"		));
		if (tempMatchCount > matches) {
			alt = 1;
			matches = tempMatchCount;
		}
		switch(alt) {
			case 1:			{
				// DEFINITION PART BEGINS (Containment)
				count = printCountingMap.get("compositionAssociationBindings");
				if (count > 0) {
					Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__COMPOSITION_ASSOCIATION_BINDINGS));
					java.util.List<?> list = (java.util.List<?>) o;
					int index = list.size() - count;
					if (index >= 0) {
						o = list.get(index);
					} else {
						o = null;
					}
					if (o != null) {
						doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
					}
					printCountingMap.put("compositionAssociationBindings", count - 1);
				}
			}
			break;
			default:			// DEFINITION PART BEGINS (Containment)
			count = printCountingMap.get("fragmentRoleBindings");
			if (count > 0) {
				Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_LANGUAGE_SPECIFICATION__FRAGMENT_ROLE_BINDINGS));
				java.util.List<?> list = (java.util.List<?>) o;
				int index = list.size() - count;
				if (index >= 0) {
					o = list.get(index);
				} else {
					o = null;
				}
				if (o != null) {
					doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
				}
				printCountingMap.put("fragmentRoleBindings", count - 1);
			}
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_RootElementContext(org.reuseware.coconut.reuseextension.RootElementContext element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(4);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__REUSE_EXTENSION));
		printCountingMap.put("reuseExtension", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_RootElementContext_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_RootElementContext_0(org.reuseware.coconut.reuseextension.RootElementContext element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ROOT_ELEMENT_CONTEXT__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentBinding(org.reuseware.coconut.reuseextension.FragmentRole2FragmentBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(4);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__FRAGMENT_ROLE));
		printCountingMap.put("fragmentRole", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__COMPONENT_MODEL_SPECIFICATION));
		printCountingMap.put("componentModelSpecification", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__PORT_TYPE_BINDINGS));
		printCountingMap.put("portTypeBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("role");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fragmentRole");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__FRAGMENT_ROLE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getFragmentRoleBindingFragmentRoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__FRAGMENT_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__FRAGMENT_ROLE), element));
				out.print(" ");
			}
			printCountingMap.put("fragmentRole", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentBinding_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("portTypeBindings");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__PORT_TYPE_BINDINGS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("portTypeBindings", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentBinding_0(org.reuseware.coconut.reuseextension.FragmentRole2FragmentBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_BINDING__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_PortType2HeterogeneousPortBinding(org.reuseware.coconut.reuseextension.PortType2HeterogeneousPortBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(3);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING__PORT_TYPE));
		printCountingMap.put("portType", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING__DERIVATION_RULES));
		printCountingMap.put("derivationRules", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING__FRAGMENT_ROLE_BINDING));
		printCountingMap.put("fragmentRoleBinding", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("type");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("portType");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING__PORT_TYPE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getPortTypeBindingPortTypeReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING__PORT_TYPE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING__PORT_TYPE), element));
			}
			printCountingMap.put("portType", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("derivationRules");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HETEROGENEOUS_PORT_BINDING__DERIVATION_RULES));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("derivationRules", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_PortType2HomogenousPortBinding(org.reuseware.coconut.reuseextension.PortType2HomogenousPortBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(3);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING__PORT_TYPE));
		printCountingMap.put("portType", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING__DERIVATION_RULES));
		printCountingMap.put("derivationRules", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING__FRAGMENT_ROLE_BINDING));
		printCountingMap.put("fragmentRoleBinding", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("homo");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("type");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("portType");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING__PORT_TYPE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getPortTypeBindingPortTypeReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING__PORT_TYPE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING__PORT_TYPE), element));
			}
			printCountingMap.put("portType", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("derivationRules");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_HOMOGENOUS_PORT_BINDING__DERIVATION_RULES));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("derivationRules", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding(org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(11);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__COMPOSITION_ASSOCIATION));
		printCountingMap.put("compositionAssociation", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__COMPOSITION_LANGUAGE_SPECIFICATION));
		printCountingMap.put("compositionLanguageSpecification", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FOR_EACH1_EXPRESSION));
		printCountingMap.put("forEach1Expression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FOR_EACH2_EXPRESSION));
		printCountingMap.put("forEach2Expression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FRAGMENT_INSTANCE1_NAME_EXPRESSION));
		printCountingMap.put("fragmentInstance1NameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FRAGMENT_INSTANCE2_NAME_EXPRESSION));
		printCountingMap.put("fragmentInstance2NameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__PORT_INSTANCE1_NAME_EXPRESSION));
		printCountingMap.put("portInstance1NameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__PORT_INSTANCE2_NAME_EXPRESSION));
		printCountingMap.put("portInstance2NameExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("association");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("compositionAssociation");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__COMPOSITION_ASSOCIATION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCompositionAssociationBindingCompositionAssociationReferenceResolver().deResolve((org.reuseware.coconut.fracol.CompositionAssociation) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__COMPOSITION_ASSOCIATION)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__COMPOSITION_ASSOCIATION), element));
			}
			printCountingMap.put("compositionAssociation", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("fragmentInstance1NameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FRAGMENT_INSTANCE1_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FRAGMENT_INSTANCE1_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("fragmentInstance1NameExpression", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("-->");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("fragmentInstance2NameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FRAGMENT_INSTANCE2_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FRAGMENT_INSTANCE2_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("fragmentInstance2NameExpression", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_0(org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_1(org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEach1Expression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FOR_EACH1_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FOR_EACH1_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEach1Expression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_2(org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portInstance1NameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__PORT_INSTANCE1_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__PORT_INSTANCE1_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portInstance1NameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_3(org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEach2Expression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FOR_EACH2_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__FOR_EACH2_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEach2Expression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_CompositionAssociation2CompositionLinkBinding_4(org.reuseware.coconut.reuseextension.CompositionAssociation2CompositionLinkBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portInstance2NameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__PORT_INSTANCE2_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.COMPOSITION_ASSOCIATION2_COMPOSITION_LINK_BINDING__PORT_INSTANCE2_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portInstance2NameExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding(org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(10);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__FRAGMENT_ROLE));
		printCountingMap.put("fragmentRole", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__COMPOSITION_LANGUAGE_SPECIFICATION));
		printCountingMap.put("compositionLanguageSpecification", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__NAME_EXPRESSION));
		printCountingMap.put("nameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__PORT_TYPE_BINDINGS));
		printCountingMap.put("portTypeBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION));
		printCountingMap.put("targetPhysicalLocationExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__TARGET_UFI_EXPRESSION));
		printCountingMap.put("targetUfiExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__UFI_EXPRESSION));
		printCountingMap.put("ufiExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("role");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fragmentRole");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__FRAGMENT_ROLE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getFragmentRoleBindingFragmentRoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__FRAGMENT_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__FRAGMENT_ROLE), element));
			}
			printCountingMap.put("fragmentRole", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("nameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("nameExpression", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("ufi");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("ufiExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__UFI_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__UFI_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("ufiExpression", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_3(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_0(org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_1(org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("target");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("ufi");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("targetUfiExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__TARGET_UFI_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__TARGET_UFI_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("targetUfiExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_2(org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("target");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("location");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("targetPhysicalLocationExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("targetPhysicalLocationExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentInstanceBinding_3(org.reuseware.coconut.reuseextension.FragmentRole2FragmentInstanceBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("portTypeBindings");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_INSTANCE_BINDING__PORT_TYPE_BINDINGS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("portTypeBindings", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding(org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(10);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__FRAGMENT_ROLE));
		printCountingMap.put("fragmentRole", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__COMPOSITION_LANGUAGE_SPECIFICATION));
		printCountingMap.put("compositionLanguageSpecification", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__NAME_EXPRESSION));
		printCountingMap.put("nameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__PORT_TYPE_BINDINGS));
		printCountingMap.put("portTypeBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION));
		printCountingMap.put("targetPhysicalLocationExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__TARGET_UFI_EXPRESSION));
		printCountingMap.put("targetUfiExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__UFI_EXPRESSION));
		printCountingMap.put("ufiExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("reference");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("role");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fragmentRole");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__FRAGMENT_ROLE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getFragmentRoleBindingFragmentRoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__FRAGMENT_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__FRAGMENT_ROLE), element));
			}
			printCountingMap.put("fragmentRole", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("nameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("nameExpression", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("ufi");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("ufiExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__UFI_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__UFI_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("ufiExpression", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("portTypeBindings");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__PORT_TYPE_BINDINGS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("portTypeBindings", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding_0(org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding_1(org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("target");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("ufi");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("targetUfiExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__TARGET_UFI_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__TARGET_UFI_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("targetUfiExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2FragmentReferenceBinding_2(org.reuseware.coconut.reuseextension.FragmentRole2FragmentReferenceBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("target");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("location");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("targetPhysicalLocationExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_FRAGMENT_REFERENCE_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("targetPhysicalLocationExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_PortType2SettingBinding(org.reuseware.coconut.reuseextension.PortType2SettingBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(3);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING__PORT_TYPE));
		printCountingMap.put("portType", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING__DERIVATION_RULES));
		printCountingMap.put("derivationRules", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING__FRAGMENT_ROLE_BINDING));
		printCountingMap.put("fragmentRoleBinding", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("portType");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING__PORT_TYPE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getPortTypeBindingPortTypeReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING__PORT_TYPE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING__PORT_TYPE), element));
			}
			printCountingMap.put("portType", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("derivationRules");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SETTING_BINDING__DERIVATION_RULES));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("derivationRules", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_SettingDerivationRule(org.reuseware.coconut.reuseextension.SettingDerivationRule element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(3);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SETTING_DERIVATION_RULE__PORT_TYPE_BINDING));
		printCountingMap.put("portTypeBinding", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SETTING_DERIVATION_RULE__PROPERTY_EXPRESSION));
		printCountingMap.put("propertyExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SETTING_DERIVATION_RULE__VALUE_EXPRESSION));
		printCountingMap.put("valueExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("propertyExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SETTING_DERIVATION_RULE__PROPERTY_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SETTING_DERIVATION_RULE__PROPERTY_EXPRESSION), element));
			}
			printCountingMap.put("propertyExpression", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("valueExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SETTING_DERIVATION_RULE__VALUE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SETTING_DERIVATION_RULE__VALUE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("valueExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(11);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__FRAGMENT_ROLE));
		printCountingMap.put("fragmentRole", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__HEIGHT_EXPRESSION));
		printCountingMap.put("heightExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__ICON_EXPRESSION));
		printCountingMap.put("iconExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__VISIBLE_EXPRESSION));
		printCountingMap.put("visibleExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__WIDTH_EXPRESSION));
		printCountingMap.put("widthExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__COMPOSITION_LANGUAGE_SYNTAX));
		printCountingMap.put("compositionLanguageSyntax", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__NAME_EXPRESSION));
		printCountingMap.put("nameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__PORT_TYPE_BINDINGS));
		printCountingMap.put("portTypeBindings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION));
		printCountingMap.put("targetPhysicalLocationExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__TARGET_UFI_EXPRESSION));
		printCountingMap.put("targetUfiExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__UFI_EXPRESSION));
		printCountingMap.put("ufiExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("role");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("fragmentRole");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__FRAGMENT_ROLE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getFragmentRoleBindingFragmentRoleReferenceResolver().deResolve((org.reuseware.coconut.fracol.FragmentRole) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__FRAGMENT_ROLE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__FRAGMENT_ROLE), element));
			}
			printCountingMap.put("fragmentRole", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_5(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_6(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_7(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("portTypeBindings");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__PORT_TYPE_BINDINGS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("portTypeBindings", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_0(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("fragment");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("nameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("nameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_1(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("ufi");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("ufiExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__UFI_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__UFI_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("ufiExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_2(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("target");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("ufi");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("targetUfiExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__TARGET_UFI_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__TARGET_UFI_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("targetUfiExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_3(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("target");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("location");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("targetPhysicalLocationExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__TARGET_PHYSICAL_LOCATION_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("targetPhysicalLocationExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_4(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("visible");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("visibleExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__VISIBLE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__VISIBLE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("visibleExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_5(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("icon");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("iconExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__ICON_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__ICON_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("iconExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_6(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("width");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("widthExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__WIDTH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__WIDTH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("widthExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_FragmentRole2SyntaxBinding_7(org.reuseware.coconut.reuseextension.FragmentRole2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("height");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("heightExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__HEIGHT_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.FRAGMENT_ROLE2_SYNTAX_BINDING__HEIGHT_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("heightExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding(org.reuseware.coconut.reuseextension.PortType2SyntaxBinding element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(7);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__HEIGHT_EXPRESSION));
		printCountingMap.put("heightExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__ICON_EXPRESSION));
		printCountingMap.put("iconExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__VISIBLE_EXPRESSION));
		printCountingMap.put("visibleExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__WIDTH_EXPRESSION));
		printCountingMap.put("widthExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__PORT_TYPE));
		printCountingMap.put("portType", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__FRAGMENT_ROLE2_SYNTAX_BINDING));
		printCountingMap.put("fragmentRole2SyntaxBinding", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__LABEL_EXPRESSION));
		printCountingMap.put("labelExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("type");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("portType");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__PORT_TYPE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getPortTypeBindingPortTypeReferenceResolver().deResolve((org.reuseware.coconut.fracol.PortType) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__PORT_TYPE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__PORT_TYPE), element));
			}
			printCountingMap.put("portType", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_0(org.reuseware.coconut.reuseextension.PortType2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("visible");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("visibleExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__VISIBLE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__VISIBLE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("visibleExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_1(org.reuseware.coconut.reuseextension.PortType2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("icon");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("iconExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__ICON_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__ICON_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("iconExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_2(org.reuseware.coconut.reuseextension.PortType2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("width");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("widthExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__WIDTH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__WIDTH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("widthExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_3(org.reuseware.coconut.reuseextension.PortType2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("height");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("heightExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__HEIGHT_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__HEIGHT_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("heightExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PortType2SyntaxBinding_4(org.reuseware.coconut.reuseextension.PortType2SyntaxBinding element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "			";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("label");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("labelExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__LABEL_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PORT_TYPE2_SYNTAX_BINDING__LABEL_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("labelExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(10);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__FOR_EACH_EXPRESSION));
		printCountingMap.put("forEachExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
		printCountingMap.put("homoPortNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__POINT_NAME_EXPRESSION));
		printCountingMap.put("pointNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__PORT_NAME_EXPRESSION));
		printCountingMap.put("portNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__PORT_TYPE_BINDING));
		printCountingMap.put("portTypeBinding", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__MODE_EXPRESSION));
		printCountingMap.put("modeExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__REMOVE_EXPRESSION));
		printCountingMap.put("removeExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("is");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("slot");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_5(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_6(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_SlotDerivationRule_7(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_0(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundFeature");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_FEATURE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundFeatureReferenceResolver().deResolve((org.eclipse.emf.ecore.EStructuralFeature) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_FEATURE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__EBOUND_FEATURE), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundFeature", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_1(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_2(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEachExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__FOR_EACH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__FOR_EACH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEachExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_3(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("mode");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("modeExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__MODE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__MODE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("modeExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_4(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("homo");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("homoPortNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("homoPortNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_5(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_6(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("point");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("pointNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__POINT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__POINT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("pointNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_SlotDerivationRule_7(org.reuseware.coconut.reuseextension.SlotDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("remove");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("removeExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__REMOVE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.SLOT_DERIVATION_RULE__REMOVE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("removeExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(10);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__FOR_EACH_EXPRESSION));
		printCountingMap.put("forEachExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
		printCountingMap.put("homoPortNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__POINT_NAME_EXPRESSION));
		printCountingMap.put("pointNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__PORT_NAME_EXPRESSION));
		printCountingMap.put("portNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__PORT_TYPE_BINDING));
		printCountingMap.put("portTypeBinding", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__MODE_EXPRESSION));
		printCountingMap.put("modeExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__REMOVE_EXPRESSION));
		printCountingMap.put("removeExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("is");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("hook");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_5(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_6(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_HookDerivationRule_7(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_0(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundFeature");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_FEATURE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundFeatureReferenceResolver().deResolve((org.eclipse.emf.ecore.EStructuralFeature) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_FEATURE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__EBOUND_FEATURE), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundFeature", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_1(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_2(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEachExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__FOR_EACH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__FOR_EACH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEachExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_3(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("mode");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("modeExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__MODE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__MODE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("modeExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_4(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("homo");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("homoPortNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("homoPortNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_5(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_6(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("point");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("pointNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__POINT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__POINT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("pointNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_HookDerivationRule_7(org.reuseware.coconut.reuseextension.HookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("remove");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("removeExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__REMOVE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.HOOK_DERIVATION_RULE__REMOVE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("removeExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_AnchorDerivationRule(org.reuseware.coconut.reuseextension.AnchorDerivationRule element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(8);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__FOR_EACH_EXPRESSION));
		printCountingMap.put("forEachExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
		printCountingMap.put("homoPortNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__POINT_NAME_EXPRESSION));
		printCountingMap.put("pointNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__PORT_NAME_EXPRESSION));
		printCountingMap.put("portNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__PORT_TYPE_BINDING));
		printCountingMap.put("portTypeBinding", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("is");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("anchor");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_5(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_0(org.reuseware.coconut.reuseextension.AnchorDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundFeature");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_FEATURE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundFeatureReferenceResolver().deResolve((org.eclipse.emf.ecore.EStructuralFeature) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_FEATURE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__EBOUND_FEATURE), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundFeature", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_1(org.reuseware.coconut.reuseextension.AnchorDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_2(org.reuseware.coconut.reuseextension.AnchorDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEachExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__FOR_EACH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__FOR_EACH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEachExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_3(org.reuseware.coconut.reuseextension.AnchorDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("homo");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("homoPortNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("homoPortNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_4(org.reuseware.coconut.reuseextension.AnchorDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_AnchorDerivationRule_5(org.reuseware.coconut.reuseextension.AnchorDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("point");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("pointNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__POINT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.ANCHOR_DERIVATION_RULE__POINT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("pointNameExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule(org.reuseware.coconut.reuseextension.PrototypeDerivationRule element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(8);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__FOR_EACH_EXPRESSION));
		printCountingMap.put("forEachExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
		printCountingMap.put("homoPortNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__POINT_NAME_EXPRESSION));
		printCountingMap.put("pointNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__PORT_NAME_EXPRESSION));
		printCountingMap.put("portNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__PORT_TYPE_BINDING));
		printCountingMap.put("portTypeBinding", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("is");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("prototype");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_5(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_0(org.reuseware.coconut.reuseextension.PrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundFeature");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundFeatureReferenceResolver().deResolve((org.eclipse.emf.ecore.EStructuralFeature) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundFeature", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_1(org.reuseware.coconut.reuseextension.PrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_2(org.reuseware.coconut.reuseextension.PrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEachExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__FOR_EACH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__FOR_EACH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEachExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_3(org.reuseware.coconut.reuseextension.PrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("homo");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("homoPortNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("homoPortNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_4(org.reuseware.coconut.reuseextension.PrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_PrototypeDerivationRule_5(org.reuseware.coconut.reuseextension.PrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("point");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("pointNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__POINT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.PROTOTYPE_DERIVATION_RULE__POINT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("pointNameExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(13);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__FOR_EACH_EXPRESSION));
		printCountingMap.put("forEachExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
		printCountingMap.put("homoPortNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__POINT_NAME_EXPRESSION));
		printCountingMap.put("pointNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__PORT_NAME_EXPRESSION));
		printCountingMap.put("portNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__PORT_TYPE_BINDING));
		printCountingMap.put("portTypeBinding", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__MODE_EXPRESSION));
		printCountingMap.put("modeExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__REMOVE_EXPRESSION));
		printCountingMap.put("removeExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__BEGIN_INDEX_EXPRESSION));
		printCountingMap.put("beginIndexExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__END_INDEX_EXPRESSION));
		printCountingMap.put("endIndexExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__LIST_INDEX_EXPRESSION));
		printCountingMap.put("listIndexExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("is");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("value");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("hook");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_5(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_6(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_7(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_8(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_9(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_10(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_0(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundFeature");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_FEATURE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundFeatureReferenceResolver().deResolve((org.eclipse.emf.ecore.EStructuralFeature) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_FEATURE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__EBOUND_FEATURE), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundFeature", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_1(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_2(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEachExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__FOR_EACH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__FOR_EACH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEachExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_3(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("mode");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("modeExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__MODE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__MODE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("modeExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_4(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("homo");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("homoPortNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("homoPortNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_5(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_6(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("point");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("pointNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__POINT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__POINT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("pointNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_7(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("begin");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("idx");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("beginIndexExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__BEGIN_INDEX_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__BEGIN_INDEX_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("beginIndexExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_8(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("end");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("idx");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("endIndexExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__END_INDEX_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__END_INDEX_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("endIndexExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_9(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("list");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("idx");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("listIndexExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__LIST_INDEX_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__LIST_INDEX_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("listIndexExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValueHookDerivationRule_10(org.reuseware.coconut.reuseextension.ValueHookDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("remove");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("removeExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__REMOVE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_HOOK_DERIVATION_RULE__REMOVE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("removeExpression", count - 1);
		}
	}
	
	
	public void print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(9);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS));
		printCountingMap.put("eBoundClass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE));
		printCountingMap.put("eBoundFeature", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__IS_EXPRESSION));
		printCountingMap.put("isExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__FOR_EACH_EXPRESSION));
		printCountingMap.put("forEachExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
		printCountingMap.put("homoPortNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__POINT_NAME_EXPRESSION));
		printCountingMap.put("pointNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__PORT_NAME_EXPRESSION));
		printCountingMap.put("portNameExpression", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__PORT_TYPE_BINDING));
		printCountingMap.put("portTypeBinding", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__VALUE_EXPRESSION));
		printCountingMap.put("valueExpression", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "		";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundClass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("is");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("value");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("prototype");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_2(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_3(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_4(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_5(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("value");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("valueExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__VALUE_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__VALUE_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("valueExpression", count - 1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_0(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(".");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingDefaultToken)
		count = printCountingMap.get("eBoundFeature");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("TEXT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRuleContextEBoundFeatureReferenceResolver().deResolve((org.eclipse.emf.ecore.EStructuralFeature) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE)), element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__EBOUND_FEATURE), element));
				out.print(" ");
			}
			printCountingMap.put("eBoundFeature", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_1(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("if");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("isExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__IS_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__IS_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("isExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_2(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("foreach");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("forEachExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__FOR_EACH_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__FOR_EACH_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("forEachExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_3(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("homo");
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("homoPortNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__HOMO_PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("homoPortNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_4(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("port");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("portNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__PORT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__PORT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("portNameExpression", count - 1);
		}
	}
	
	public void print_org_reuseware_coconut_reuseextension_ValuePrototypeDerivationRule_5(org.reuseware.coconut.reuseextension.ValuePrototypeDerivationRule element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("point");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("=");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("pointNameExpression");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__POINT_NAME_EXPRESSION));
			if (o != null) {
				org.reuseware.coconut.reuseextension.resource.rex.IRexTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_36_36");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.reuseware.coconut.reuseextension.ReuseextensionPackage.VALUE_PROTOTYPE_DERIVATION_RULE__POINT_NAME_EXPRESSION), element));
				out.print(" ");
			}
			printCountingMap.put("pointNameExpression", count - 1);
		}
	}
	
	
}
